#include ".\strmanger.h"
#include <windows.h>
#include <string>
#include <vector>
#include <CommCtrl.h>

NullStrSrc CStrManger::m_nullstrsrc;

CStrManger::CStrManger()
{
	m_pStrSrc = &m_nullstrsrc;
	m_autodel = false;
	m_revupte = false;
}

CStrManger::~CStrManger(void)
{
	if(m_autodel)
	{
		delete m_pStrSrc;
	}
}

void  CStrManger::SetSource(IStrSrc * pSource, bool autodel)
{
	if(m_autodel)
		delete m_pStrSrc;

	m_pStrSrc = pSource;
	m_autodel = autodel;
}

const std::string& CStrManger::GetString(const std::string& sid)
{
	return m_pStrSrc->GetString(sid);
}

const std::string& CStrManger::GetString(unsigned int sid)
{
	static char keybuffer[STR_BUFSIZE];
	sprintf_s(keybuffer, STR_BUFSIZE, "%d", sid);
	return m_pStrSrc->GetString(keybuffer);
}

void CStrManger::UpdateMenuItem(HMENU hMenu, int pos)
{
	static char menustr[STR_BUFSIZE];
	MENUITEMINFO menuiteminfo;
	menuiteminfo.cbSize = sizeof(MENUITEMINFO);
	menuiteminfo.fMask =  MIIM_SUBMENU | MIIM_TYPE;
	menuiteminfo.fType = MFT_STRING;
	menuiteminfo.dwTypeData = menustr;

	if(GetMenuItemInfo(hMenu, pos, TRUE, &menuiteminfo))
	{
		std::string title = "";
		if(m_revupte)
			title = m_pStrSrc->GetKey(menustr);
		else
			title = GetString(menustr);

		if(title != "")
		{
			menuiteminfo.fMask = MIIM_TYPE;
			strncpy_s(menustr, STR_BUFSIZE, title.c_str(), STR_BUFSIZE);
			SetMenuItemInfo(hMenu, pos, TRUE, &menuiteminfo);
		}

		if(menuiteminfo.hSubMenu != NULL)
		{
			UpdateMenus(menuiteminfo.hSubMenu);
		}
	}
}

void CStrManger::UpdateMenus(HMENU hMenu)
{
	int itemcnt = GetMenuItemCount(hMenu);

	for(int i = 0; i < itemcnt; ++i)
	{
		UpdateMenuItem(hMenu, i);
	}
}

void CStrManger::UpdateControl(HWND hWnd)
{
	EnumChildWindows(hWnd, EnumChildProc, reinterpret_cast<LPARAM>(this));
}

void CStrManger::UnUpdateControl(HWND hWnd)
{
	m_revupte = true;
	UpdateControl(hWnd);
	m_revupte = false;
}

void CStrManger::UnUpdateMenus(HMENU hMenu)
{
	m_revupte = true;
	UpdateMenus(hMenu);
	m_revupte = false;
}

BOOL CALLBACK CStrManger::EnumChildProc(HWND hwndChild, LPARAM lParam)
{
	static char buffer[STR_BUFSIZE];
	CStrManger * pManger = reinterpret_cast<CStrManger *>(lParam);
	GetWindowText(hwndChild, buffer, STR_BUFSIZE);
	std::string title = "";

	if(pManger->m_revupte)
		title = pManger->m_pStrSrc->GetKey(buffer);
	else
		title = pManger->GetString(buffer);

	if(title != "")
	{
		SetWindowText(hwndChild, title.c_str());
	}


    GetClassName(hwndChild, buffer, STR_BUFSIZE);

    static std::string combox = "ComboBox";
    if(combox == buffer)
    {
        std::vector<std::string>  m_lists;
        int nCount = (int)SendMessage(hwndChild, CB_GETCOUNT, 0, 0);
        int nCurSel = (int)SendMessage(hwndChild, CB_GETCURSEL, 0, 0);

        for(int i = 0; i < nCount; ++i)
        {
            SendMessage(hwndChild, CB_GETLBTEXT, (WPARAM)i, (LPARAM)buffer);

            if(pManger->m_revupte)
                title = pManger->m_pStrSrc->GetKey(buffer);
            else
                title = pManger->GetString(buffer);

            if(title == "")
            {
                title = buffer;
            }

            m_lists.push_back(title);
        }

        SendMessage(hwndChild, CB_RESETCONTENT, 0, 0);
        for(int i = 0; i < nCount; ++i)
        {
            SendMessage(hwndChild, CB_ADDSTRING, 0, (LPARAM)m_lists[i].c_str());
        }
        SendMessage(hwndChild, CB_SETCURSEL, (WPARAM)nCurSel, 0);
    }

    static std::string tabbox = "SysTabControl32";
    if(tabbox == buffer)
    {
        int nCount = TabCtrl_GetItemCount(hwndChild);

        TCITEM  item;
        item.mask = TCIF_TEXT;
        item.pszText = buffer;
        item.cchTextMax = STR_BUFSIZE;

        for(int i = 0; i < nCount; ++i)
        {
            TabCtrl_GetItem(hwndChild, i, &item);

            if(pManger->m_revupte)
                title = pManger->m_pStrSrc->GetKey(buffer);
            else
                title = pManger->GetString(buffer);

            if(title != "")
            {
                strncpy_s(buffer, STR_BUFSIZE, title.c_str(), STR_BUFSIZE);
                TabCtrl_SetItem(hwndChild, i, &item);
            }
        }
    }
  
	return true;
}